-----------------------------------------------------------------------------------------------
-- Client Lua Script for FrostMod_ThreatBall
-- Author: Frosthaven
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------

require "Apollo"
require "GameLib"
require "Window"

-- DEFINITION ---------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------
local FMTB = {
	uiMapperLib = "uiMapper:0.9",
	--meta data
	meta = {
		name        = "FrostMod ThreatBall",
		version     = "1.5",
		author      = "Frosthaven",
	},
	--debuffs checked for taunt status
	taunts = {
		["Reaver"]       = true,
		["Plasma Blast"] = true,
		["Blitz"]        = true,
		["Code Red"]     = true,
	},
	--decides if in edit mode
	moving       = false,
}

-- INITIALIZATION -----------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------
function FMTB:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    return o
end

function FMTB:Init()
	local bHasConfigureFunction = true
	local strConfigureButtonText = "FrostMod Threatball"
	local tDependencies = {
		self.uiMapperLib,
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end

function FMTB:OnConfigure()
	if self.ui then
		self.ui.wndMain:Show(true, true)
	end
end

-- ADDON LOADED -------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------
function FMTB:OnLoad()
	-- load default settings -------------------------------------------
	--------------------------------------------------------------------
	self.saved    = self:GetDefaults()

	-- load our forms --------------------------------------------------
	--------------------------------------------------------------------
	self.xmlDoc = XmlDoc.CreateFromFile("form.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)

	-- initialize our ui -----------------------------------------------
	--------------------------------------------------------------------
	local uiMapper = Apollo.GetPackage(self.uiMapperLib).tPackage
	self.ui  = uiMapper:new({
		container = self.saved,
		defaults  = self:GetDefaults(),
		name      = self.meta.name,
		author    = self.meta.author,
		version   = self.meta.version,
		slash     = "fmtb",
		onshow    = function(ui)
			--config panel shown
			self:ToggleEditMode()
		end,
		onhide    = function(ui)
			--config panel hidden
			self:ToggleEditMode()
		end,
		ondefault = function(ui)
			--default settings have been applied
		end,
	}):build(function(ui)
		self:BuildUI(ui)
	end)
end

-- FORM LOADED --------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------
function FMTB:OnDocLoaded()
	-- load sprite sheet -----------------------------------------------
	--------------------------------------------------------------------
	Apollo.LoadSprites("textures/sprites.xml", "FrostMod_ThreatBall_Sprites")

	-- process loaded form ---------------------------------------------
	--------------------------------------------------------------------
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "ring", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the threat ball for some reason.")
			return
		end

		-- register events -------------------------------------------------
		--------------------------------------------------------------------
		--slash command events
		--Apollo.RegisterSlashCommand("fmtb", "OnCmd", self)

		--game events
		Apollo.RegisterEventHandler("TargetThreatListUpdated", "OnThreatUpdated", self)
		Apollo.RegisterEventHandler("TargetUnitChanged",       "OnChangedTarget", self)

		-- initialize the threat ball --------------------------------------
		--------------------------------------------------------------------
		--progress bar
		--self.wndMain:FindChild("progress"):SetRotation(-135) -- rotation causes display glitch
		self.wndMain:FindChild("progress"):SetFloor(0)
		self.wndMain:FindChild("progress"):SetMax(100)
		self.wndMain:FindChild("progress"):SetBGColor(self.saved.bar_low)

		--threat level text
		self.wndMain:FindChild("level"):SetTextColor(self.saved.txt_low)
		self.wndMain:FindChild("level"):SetSprite("")
		self.wndMain:FindChild("level"):SetBGOpacity(0.4)
	    self.wndMain:Show(false, true)

	    --ball style
	    self:UpdateBallStyle(self.saved.ballStyle)
		self.wndMain:SetBGOpacity(self.saved.artOpacity/100)
		self.wndMain:FindChild("inner"):SetBGOpacity(self.saved.artOpacity/100)

	    --unit name
	    self:UpdateThreatCompetitionName("Example Name")
	    self.wndMain:FindChild("unitname"):SetTextColor(self.saved.tname_notank)
		self.wndMain:FindChild("unitname"):SetSprite("")
		self.wndMain:FindChild("unitname"):SetBGOpacity(0.4)

		-- restore the window positions
		self.defaultNameAnchors  = {self.wndMain:FindChild("unitname"):GetAnchorPoints()}
		self.defaultNameOffsets  = {self.wndMain:FindChild("unitname"):GetAnchorOffsets()}
		self.defaultLevelAnchors = {self.wndMain:FindChild("level"):GetAnchorPoints()}
		self.defaultLevelOffsets = {self.wndMain:FindChild("level"):GetAnchorOffsets()}
		self:AdjustLayerSizes()

		if (self.saved.wndOffsets ~= nil and self.saved.wndAnchors ~= nil) then
			self.wndMain:SetAnchorPoints(unpack(self.saved.wndAnchors))
			self.wndMain:SetAnchorOffsets(unpack(self.saved.wndOffsets))
		end
		if (self.saved.allowLevelMove and self.saved.levelOffsets ~= nil and self.saved.levelAnchors ~= nil) then
			self.wndMain:FindChild("level"):SetAnchorPoints(unpack(self.saved.levelAnchors))
			self.wndMain:FindChild("level"):SetAnchorOffsets(unpack(self.saved.levelOffsets))
		end
		if (self.saved.allowNameMove and self.saved.nameOffsets ~= nil and self.saved.nameAnchors ~= nil) then
			self.wndMain:FindChild("unitname"):SetAnchorPoints(unpack(self.saved.nameAnchors))
			self.wndMain:FindChild("unitname"):SetAnchorOffsets(unpack(self.saved.nameOffsets))
		end
	end
end

-- SAVED USER SETTINGS ------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------
function FMTB:GetDefaults()
	return {
		-- window constraints are set from edit mode
		wndOffsets   = nil,
		wndAnchors   = nil,

		-- overall styles
		scale = 0.8,
		ballStyle      = "default", --default--full--holo
		artOpacity     = 100,
		levelFont      = "CRB_Header14_O",
		nameFont       = "CRB_Interface10_BBO",
		levelScale     = 1,
		nameScale      = 1,
		allowLevelMove = false,
		allowNameMove  = false,

		-- thresholds for changing styles
		lThreshold   = 30,
		hThreshold   = 88,

		-- next highest threat unit
		highThreatPlayerDisplay = "first", --false--full--abbr--first
		
		-- bar colors
		bar_low     = {0.25, 0.36, 0.36, 1},
		bar_normal  = {0.25, 0.36, 0.36, 1},
		bar_high    = {0.68, 0.36, 0.00, 1},
		bar_tanking = {0.53, 0.00, 0.00, 1},
		bar_taunted = {0.43, 0.00, 0.58, 1},

		-- text colors
		tname_tank   = {1.00, 1.00, 1.00, 1},
		tname_notank = {1.00, 1.00, 1.00, 1},
		txt_low      = {1.00, 1.00, 1.00, 1},
		txt_normal   = {1.00, 1.00, 1.00, 1},
		txt_high     = {1.00, 0.71, 0.38, 1},
		txt_tanking  = {1.00, 0.50, 0.50, 1},
		txt_taunted  = {1.00, 0.52, 1.00, 1},
	}
end

function FMTB:OnSave(eLevel)
	if (eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character) then
		return nil
	end
	
	--preload our window sizes and positions
	--ball
	self.saved.wndAnchors   = {self.wndMain:GetAnchorPoints()}
	self.saved.wndOffsets   = {self.wndMain:GetAnchorOffsets()}
	--level
	self.saved.levelAnchors = {self.wndMain:FindChild("level"):GetAnchorPoints()}
	self.saved.levelOffsets = {self.wndMain:FindChild("level"):GetAnchorOffsets()}
	--name
	self.saved.nameAnchors  = {self.wndMain:FindChild("unitname"):GetAnchorPoints()}
	self.saved.nameOffsets = {self.wndMain:FindChild("unitname"):GetAnchorOffsets()}

	--save the entire config area
	return self.saved
end

function FMTB:OnRestore(eLevel, savedData)
	if (eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character) then
		return nil
	end

	for k, v in pairs(self.saved) do
		if savedData[k] ~= nil then
			self.saved[k] = savedData[k]
		end
	end
	
	self.saved.wndAnchors   = savedData.wndAnchors
	self.saved.wndOffsets   = savedData.wndOffsets
	self.saved.levelAnchors = savedData.levelAnchors
	self.saved.levelOffsets = savedData.levelOffsets
	self.saved.nameAnchors  = savedData.nameAnchors
	self.saved.nameOffsets  = savedData.nameOffsets
end

-- GAME EVENTS --------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------
function FMTB:AdjustLayerSizes()
	--todo
	--[[
	self.ui:log("adjusting sizes")

	local level = self.wndMain:FindChild("level")
	local name  = self.wndMain:FindChild("unitname")

	self.ui:log("width is " .. level:GetWidth())
	--]]
end

function FMTB:ToggleEditMode()
	--toggles moving/editing mode for the ring
	self.moving = not self.moving
	self.wndMain:Show(self.moving, true)
	self.wndMain:FindChild("level"):SetText("move")
	self.wndMain:FindChild("level"):SetTextColor(self.saved.txt_low)

	self.wndMain:SetStyle("Moveable", self.moving)

	local sprite
	if self.moving then
		self.wndMain:FindChild("ring"):SetStyle("IgnoreMouse", false)
		self.wndMain:FindChild("level"):SetStyle("Moveable", self.saved.allowLevelMove)
		self.wndMain:FindChild("level"):SetStyle("IgnoreMouse", not self.saved.allowLevelMove)
		self.wndMain:FindChild("level"):SetStyle("Sizable", self.saved.allowLevelMove)
		self.wndMain:FindChild("unitname"):SetStyle("Moveable", self.saved.allowNameMove)
		self.wndMain:FindChild("unitname"):SetStyle("IgnoreMouse", not self.saved.allowNameMove)
		self.wndMain:FindChild("unitname"):SetStyle("Sizable", self.saved.allowNameMove)
		
		sprite = self.saved.allowLevelMove and "CRB_ActionBarIconSprites:sprActionBar_OrangeBorder" or ""
		self.wndMain:FindChild("level"):SetSprite(sprite)
		
		sprite = self.saved.allowNameMove and "CRB_ActionBarIconSprites:sprActionBar_OrangeBorder" or ""
		self.wndMain:FindChild("unitname"):SetSprite(sprite)
	else
		self.wndMain:FindChild("ring"):SetStyle("IgnoreMouse", true)
		self.wndMain:FindChild("level"):SetStyle("Moveable", false)
		self.wndMain:FindChild("level"):SetStyle("IgnoreMouse", true)
		self.wndMain:FindChild("level"):SetStyle("Sizable", false)
		self.wndMain:FindChild("level"):SetSprite("")
		self.wndMain:FindChild("unitname"):SetStyle("Moveable", false)
		self.wndMain:FindChild("unitname"):SetStyle("IgnoreMouse", true)
		self.wndMain:FindChild("unitname"):SetStyle("Sizable", false)
		self.wndMain:FindChild("unitname"):SetSprite("")
	end

	self.wndMain:FindChild("progress"):SetProgress(75)
	self.wndMain:FindChild("progress"):SetBGColor(self.saved.bar_low)
end

function FMTB:OnChangedTarget(unitTarget)
	--hide the threat meter on deselects
	if not self.moving then
		self.wndMain:Show(false, true)
	end
end

function FMTB:OnThreatUpdated(...)
	--threat tables have changed, let's analyze them
	if select(1, ...) ~= nil then
		local topThreatUnit = select(1, ...)
		local topThreatValue = select(2, ...)
		if topThreatUnit == GameLib.GetPlayerUnit() then
			-- player has aggro
			if select(3, ...) ~= nil then
				--there are other units competing for threat
				--we care about the person gaining on us
				local threatLead = select(4, ...) / topThreatValue * 100
				self:ProcessThreatAsTank(threatLead, select(3, ...))
			else
				--there is no threat competition
				self.wndMain:Show(false, true)
				return
			end
		else
			-- someone else has aggro
			self:ProcessThreatAsDPS(topThreatValue, topThreatUnit, ...)
		end
		self:CheckTauntStatus()
	end
end

-- ADDON FUNCTIONS ----------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------
function FMTB:UpdateThreatCompetitionName(name)
	--set the option
	if self.saved.highThreatPlayerDisplay then
		--split the name into first/last
		local names = {}
		for w in name:gmatch("%S+") do
			table.insert(names, w)
		end

		if self.saved.highThreatPlayerDisplay == 'full' then
			--check for last name
			if names[2] then
				name = names[1] .. "\n" .. names[2]
			else
				name = "\n" .. names[1]
			end
		elseif self.saved.highThreatPlayerDisplay == 'first' then
			name = "\n" .. names[1]
		elseif self.saved.highThreatPlayerDisplay == 'abbr' then
			--check for last name
			if names[2] then
				name = "\n" .. names[1]:sub(1,3) .. names[2]:sub(1,3)
			else
				name = "\n" .. names[1]:sub(1,3)
			end
		end
		self.wndMain:FindChild("unitname"):SetText(name)
		self.wndMain:FindChild("unitname"):Show(true, true)
	else
		self.wndMain:FindChild("unitname"):Show(false, true)
	end
end

function FMTB:UpdateRingStyle(style, text)
	-- style    : string : low normal high tanking taunted
	-- text     : string

	self.wndMain:FindChild("progress"):SetBGColor(self.saved['bar_' .. style])
	self.wndMain:FindChild("level"):SetTextColor(self.saved['txt_' .. style])

	if style == "tanking" then
		self.wndMain:FindChild("unitname"):SetTextColor(self.saved.tname_tank)
	else
		self.wndMain:FindChild("unitname"):SetTextColor(self.saved.tname_notank)
	end

	if text then
		self.wndMain:FindChild("level"):SetText(tostring(text))
		self.wndMain:FindChild("progress"):SetProgress(tonumber(text))
	end
end

function FMTB:UpdateBallStyle(style)
	local outer  = self.wndMain
	local inner  = outer:FindChild("inner")
	local level  = outer:FindChild("level")
	local name   = outer:FindChild("unitname")
	local bar    = outer:FindChild("progress")

	outer:SetScale(self.saved.scale)
	level:SetFont(self.saved.levelFont)
	level:SetScale(self.saved.levelScale)
	name:SetFont(self.saved.nameFont)
	name:SetScale(self.saved.nameScale)

	if style == "text" then
		outer:SetSprite("")
		inner:SetSprite("")
		bar:SetSprite("")
		bar:SetFullSprite("")
		inner:Show(false, true)
	else
		bar:SetBGOpacity(1)

		if style == "default" then
			outer:SetSprite("FrostMod_ThreatBall_Sprites:ring")
			inner:SetSprite("FrostMod_ThreatBall_Sprites:inner")
			bar:SetFullSprite("FrostMod_ThreatBall_Sprites:bar")
		    inner:Show(true,true)
		elseif style == "full" then
			outer:SetSprite("FrostMod_ThreatBall_Sprites:ring")
			bar:SetFullSprite("FrostMod_ThreatBall_Sprites:bar")
		    inner:Show(false,true)
		elseif style == "holo" then
			outer:SetSprite("")
			bar:SetFullSprite("FrostMod_ThreatBall_Sprites:bar_holo")
		    inner:Show(false,true)
		elseif style == "holo_o" then
			outer:SetSprite("FrostMod_ThreatBall_Sprites:ring_outline")
			bar:SetFullSprite("FrostMod_ThreatBall_Sprites:bar_holo")
		    inner:Show(false,true)
		end
	end
end

function FMTB:CheckTauntStatus()
	local debuffs = GameLib.GetTargetUnit():GetBuffs().arHarmful
	local debuffName

	for _, debuff in pairs(debuffs) do
		debuffName = debuff.splEffect:GetName()
		if self.taunts[debuffName] then
			-- we need to update our style to be the taunt style
			self:UpdateRingStyle('taunted')
			break
		end
	end
end

function FMTB:ProcessThreatAsTank(nThreat, threatUnit)
	-- lets get the threat percentage lead we have. nThreat here is the percentage of
	-- the person next on the threat table.
	nThreat = 100 - nThreat;

	local text = ""
	if nThreat <= 0 then
		--negative threat happens in transitional periods when losing aggro
		text = "+0"
	else
		text = "+" .. string.format("%d", nThreat)
	end

	--set our ring and text colors, and show the tanking emblem
	self:UpdateRingStyle("tanking", text)
	self:UpdateThreatCompetitionName(threatUnit:GetName())
	self.wndMain:Show(true, true)
end

function FMTB:ProcessThreatAsDPS(threatLead, threatUnit, ...)
	local found = false
	-- Where are we at in the threat tables?
	for i=3, select('#', ...), 2 do
		local cUnit = select(i, ...)
		local cThreat = select(i+1, ...)

		if cUnit ~= nil and cUnit == GameLib.GetPlayerUnit() then
			local val = cThreat / threatLead * 100
			if (val > 200 or val < 0) then
				--sometimes numbers overflow and cause erroneous threat data, so we
				--dont process anything in those cases
			else
				if val >= tonumber(self.saved.hThreshold) then
					self:UpdateRingStyle("high", string.format("%d", val))
				elseif val <= tonumber(self.saved.lThreshold) then
					self:UpdateRingStyle("low", string.format("%d", val))
				else
					self:UpdateRingStyle("normal", string.format("%d", val))
				end
			end
			found = true
			break
		end
	end

	if not found then
		--we werent high enough threat to show in the threat list at all
		self:UpdateRingStyle("low", "0")
	end
	self:UpdateThreatCompetitionName(threatUnit:GetName())
	self.wndMain:Show(true, true)
end
-- HELPER FUNCTIONS ---------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------
function FMTB:str_split(text, sep)
    local sep, fields = sep or ":", {}
    local pattern = string.format("([^%s]+)", sep)
    self:gsub(pattern, function(c) fields[#fields+1] = c end)
    return fields
end

-- CREATE INSTANCE ----------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------
local FMTBInst = FMTB:new()
FMTBInst:Init()
