local FMTB = Apollo.GetAddon("FrostMod_ThreatBall")

function FMTB:BuildUI(ui)
	-- choice tables -----------------------
	----------------------------------------
	--ball style
	ui:choicetable("ball_style", {
		{"Default Style",     "default"},
		{"Full Pie Circle",   "full"   },
		{"Holo Ring",         "holo"   },
		{"Holo Ring Outline", "holo_o" },
		{"Text Only",         "text" },
	})
	--threat competition display
	ui:choicetable("threat_competition", {
		{"Do not display",       false  },
		{"Full name",            "full" },
		{"First name",           "first"},
		{"Abbreviated (FirLas)", "abbr" },
	})
	--fonts
	local fonts = {}
	local fontlib = Apollo:GetGameFonts()
	for k, v in pairs(fontlib) do
		table.insert(fonts, {v.name})
	end
	ui:choicetable("fonts", fonts)

	-- interface ---------------------------
	----------------------------------------
	ui:category("Ball Settings")
	:header("Style")
	:note("While this config panel is open you can preview and move the threat ball")
	:combo({
		label    = "Threatball Style",
		map      = "ballStyle",
		choices  = "ball_style",
		onchange = function(wnd)
			self:UpdateBallStyle(self.saved.ballStyle)
		end,
	})
	:slider({
		label = "Threatball Art Opacity",
		map   = "artOpacity",
		range = "0,100",
		onchange = function(wnd)
			self.wndMain:SetBGOpacity(self.saved.artOpacity/100)
			self.wndMain:FindChild("inner"):SetBGOpacity(self.saved.artOpacity/100)
		end,
	})
	:input({
		label = "Threat Ball Scale",
		map   = "scale",
		onchange = function()
			self.wndMain:SetScale(tonumber(self.saved.scale))
		end,
	})
	ui:header("Thresholds")
	:input({
		label = "Low Threat When     <",
		map   = "lThreshold",
	})
	:input({
		label = "High Threat When     >",
		map   = "hThreshold",
	})
	:header("Threat Competition Unit Text Colors")
	:note("If you are tanking, this will show closest threat\nIf you are not tanking, this will show who is")
	:combo({
		label = "Name Format",
		choices = "threat_competition",
		map   = "highThreatPlayerDisplay",
		onchange = function()
			self:UpdateThreatCompetitionName(GameLib.GetPlayerUnit():GetName())
		end,
	})
	:color({
		label = "When I'm Tanking",
		map   = "tname_tank",
		dec   = true,
		alpha = true,
		onchange = function()
			self.wndMain:FindChild("unitname"):SetTextColor(self.saved.tname_tank)
		end,
	})
	:color({
		label = "When Someone Else Is Tanking",
		map   = "tname_notank",
		dec   = true,
		alpha = true,
		onchange = function()
			self.wndMain:FindChild("unitname"):SetTextColor(self.saved.tname_notank)
		end,
	})
	:header("Low Threat")
	:color({
		label = "Text",
		map   = "txt_low",
		dec   = true,
		alpha = true,
		onchange = function()
			self.wndMain:FindChild("level"):SetTextColor(self.saved.txt_low)
			self.wndMain:FindChild("progress"):SetBGColor(self.saved.bar_low)
		end,
	})
	:color({
		label = "Meter",
		map   = "bar_low",
		dec   = true,
		alpha = true,
		onchange = function()
			self.wndMain:FindChild("level"):SetTextColor(self.saved.txt_low)
			self.wndMain:FindChild("progress"):SetBGColor(self.saved.bar_low)
		end,
	})
	:header("Normal Threat")
	:color({
		label = "Text",
		map   = "txt_normal",
		dec   = true,
		alpha = true,
		onchange = function()
			self.wndMain:FindChild("level"):SetTextColor(self.saved.txt_normal)
			self.wndMain:FindChild("progress"):SetBGColor(self.saved.bar_normal)
		end,
	})
	:color({
		label = "Meter",
		map   = "bar_normal",
		dec   = true,
		alpha = true,
		onchange = function()
			self.wndMain:FindChild("level"):SetTextColor(self.saved.txt_normal)
			self.wndMain:FindChild("progress"):SetBGColor(self.saved.bar_normal)
		end,
	})
	:header("High Threat")
	:color({
		label = "Text",
		map   = "txt_high",
		dec   = true,
		alpha = true,
		onchange = function()
			self.wndMain:FindChild("level"):SetTextColor(self.saved.txt_high)
			self.wndMain:FindChild("progress"):SetBGColor(self.saved.bar_high)
		end,
	})
	:color({
		label = "Meter",
		map   = "bar_high",
		dec   = true,
		alpha = true,
		onchange = function()
			self.wndMain:FindChild("level"):SetTextColor(self.saved.txt_high)
			self.wndMain:FindChild("progress"):SetBGColor(self.saved.bar_high)
		end,
	})
	:header("Tanking")
	:color({
		label = "Text",
		map   = "txt_tanking",
		dec   = true,
		alpha = true,
		onchange = function()
			self.wndMain:FindChild("level"):SetTextColor(self.saved.txt_tanking)
			self.wndMain:FindChild("progress"):SetBGColor(self.saved.bar_tanking)
		end,
	})
	:color({
		label = "Meter",
		map   = "bar_tanking",
		dec   = true,
		alpha = true,
		onchange = function()
			self.wndMain:FindChild("level"):SetTextColor(self.saved.txt_tanking)
			self.wndMain:FindChild("progress"):SetBGColor(self.saved.bar_tanking)
		end,
	})
	:header("Taunted")
	:color({
		label = "Text",
		map   = "txt_taunted",
		dec   = true,
		alpha = true,
		onchange = function()
			self.wndMain:FindChild("level"):SetTextColor(self.saved.txt_taunted)
			self.wndMain:FindChild("progress"):SetBGColor(self.saved.bar_taunted)
		end,
	})
	:color({
		label = "Meter",
		map   = "bar_taunted",
		dec   = true,
		alpha = true,
		onchange = function()
			self.wndMain:FindChild("level"):SetTextColor(self.saved.txt_taunted)
			self.wndMain:FindChild("progress"):SetBGColor(self.saved.bar_taunted)
		end,
	})
	:navdivider()

	:category("Advanced Font Settings")
	:header("Before You Edit")
	:note("These are advanced options that modify the font stylings and layout for the threat ball.\n \nWhen you enable a custom location and size for each text layer, a green border will show up around the text layer while the configuration panel is open. Unchecking custom location for a text layer will return it to it's initial location and size.\n \nLarger fonts may require you to resize the container so that the text can fit without clipping.\n \nMoving:   Drag the container with your mouse\nResizing: Drag from the edge of a container with your mouse")
	:header("Threat Percentage Text")
	:check({
		label = "Enable custom location & size for threat percent text",
		map   = "allowLevelMove",
		onchange = function(wnd)
			self.wndMain:FindChild("level"):SetStyle("Moveable", self.saved.allowLevelMove)
			self.wndMain:FindChild("level"):SetStyle("IgnoreMouse", not self.saved.allowLevelMove)
			self.wndMain:FindChild("level"):SetStyle("Sizable", self.saved.allowLevelMove)

			local sprite = self.saved.allowLevelMove and "CRB_ActionBarIconSprites:sprActionBar_OrangeBorder" or ""
			self.wndMain:FindChild("level"):SetSprite(sprite)

			if not self.saved.allowLevelMove then
				self.wndMain:FindChild("level"):SetAnchorPoints(unpack(self.defaultLevelAnchors))
				self.wndMain:FindChild("level"):SetAnchorOffsets(unpack(self.defaultLevelOffsets))
			end
		end,
	})
	:combo({
		label   = "Font Family",
		map     = "levelFont",
		choices = "fonts",
		onitemadded = function(wndControl)
			wndControl:SetFont(wndControl:GetText())
		end,
		onchange = function()
			self:UpdateBallStyle(self.saved.ballStyle)
		end,
	})
	:input({
		label = "Container Scale",
		map   = "levelScale",
		onchange = function()
			self.wndMain:FindChild("level"):SetScale(self.saved.levelScale)
		end,
	})
	:header("Threat Competition Unit Text")
	:check({
		label = "Enable custom location & size for threat unit text",
		map   = "allowNameMove",
		onchange = function(wnd)
			self.wndMain:FindChild("unitname"):SetStyle("Moveable", self.saved.allowNameMove)
			self.wndMain:FindChild("unitname"):SetStyle("IgnoreMouse", not self.saved.allowNameMove)
			self.wndMain:FindChild("unitname"):SetStyle("Sizable", self.saved.allowNameMove)

			local sprite = self.saved.allowNameMove and "CRB_ActionBarIconSprites:sprActionBar_OrangeBorder" or ""
			self.wndMain:FindChild("unitname"):SetSprite(sprite)

			if not self.saved.allowNameMove then
				self.wndMain:FindChild("unitname"):SetAnchorPoints(unpack(self.defaultNameAnchors))
				self.wndMain:FindChild("unitname"):SetAnchorOffsets(unpack(self.defaultNameOffsets))
			end
		end,
	})
	:combo({
		label   = "Font Family",
		map     = "nameFont",
		choices = "fonts",
		onitemadded = function(wndControl)
			wndControl:SetFont(wndControl:GetText())
		end,
		onchange = function()
			self:UpdateBallStyle(self.saved.ballStyle)
		end,
	})
	:input({
		label = "Container Scale",
		map   = "nameScale",
		onchange = function()
			self.wndMain:FindChild("unitname"):SetScale(self.saved.nameScale)
		end,
	})
end